<?php

namespace RP\AdsManager\Backend;

use RP\AdsManager\Common\wpView;
use RP\AdsManager\Database;
use RP\AdsManager\Executable;
use RP\AdsManager\Hookable;
use RP\AdsManager\Runable;
use RP\AdsManager\Singleton;

class Controller extends Singleton {
	use Runable, Executable, Hookable;

	public function init() {
		$this->registerHooks( dirname( __FILE__ ) );

		return $this;
	}

	public function listAction() {
		global $wpdb;

		$ads = $wpdb->prefix . Database::ADS_TABLE;
		$rel = $wpdb->prefix . Database::REL_TABLE;

		$wpView = wpView::getInstance();

		$records = $wpdb->get_results(
			"SELECT `a`.`id`, `a`.`title` AS `atitle`, `a`.`init`, `a`.`type`, `a`.`enabled`, `r`.`options`, `r`.`ple_id`
              FROM $ads AS `a` 
              LEFT JOIN $rel AS `r` ON `r`.`ads_id` = `a`.`id`
              ORDER BY `a`.`id`",
			ARRAY_A
		);

		$ads = [];

		foreach ( $records as $record ) {
			if ( array_key_exists( $record['id'], $ads ) ) {
				$ads[ (int) $record['id'] ]['places'][] = [
					'title'   => $wpView->getSlug( $record['ple_id'], true ),
					'options' => $record['options'],
				];
			} else {
				$ads[ (int) $record['id'] ] = [
					'init'    => ! is_null( $record['init'] ),
					'title'   => $record['atitle'],
					'type'    => $record['type'],
					'enabled' => (bool) $record['enabled'],
					'places'  => [],
				];

				if ( $record['type'] == 'ads' ) {
					$ads[ (int) $record['id'] ]['places'][] = [
						'title'   => $wpView->getSlug( $record['ple_id'], true ),
						'options' => $record['options'],
					];
				}
			}
		}

		return [
			'updated' => get_transient( 'rpam_quick_action' ),
			'records' => $ads,
			'types'   => [
				'wgt' => 'Widget',
				'ads' => 'Block',
			],
		];
	}

	public static function processPost() {
		global $pagenow;

		$requestedPage = filter_input( INPUT_GET, 'page', FILTER_SANITIZE_STRING );

		if ( $pagenow !== 'tools.php' || $requestedPage !== 'rp-ads-manager-plugin' ) {
			return false;
		}

		$action = filter_input( INPUT_GET, 'action', FILTER_SANITIZE_STRING );
		$id     = filter_input( INPUT_GET, 'id', FILTER_SANITIZE_NUMBER_INT );

		$type = filter_input( INPUT_GET, 'type', FILTER_SANITIZE_STRING );

		if ( $action == 'editor' && ! in_array( $type, [ 'wgt', 'ads' ] ) ) {
			header( 'Location: ' . admin_url( 'tools.php?page=rp-ads-manager-plugin' ) );
			exit;
		}

		global $wpdb;

		if ( in_array( $action, [ 'enable', 'disable', 'delete' ] ) && null !== $id ) {

			switch ( $action ) {
				case 'delete':
					$wpdb->delete( $wpdb->prefix . Database::ADS_TABLE,
						[
							'id' => $id,
						]
					);
					$wpdb->delete( $wpdb->prefix . Database::REL_TABLE,
						[
							'ads_id' => $id,
						]
					);

					set_transient( 'rpam_quick_action', __('Object successfully deleted', 'rp-ads-manager'), 2 );

					break;
				case 'enable':
					$wpdb->update( $wpdb->prefix . Database::ADS_TABLE,
						[
							'enabled' => 1,
						],
						[
							'id' => $id,
						]
					);

					set_transient( 'rpam_quick_action', __('Object successfully enabled', 'rp-ads-manager'), 2 );
					break;
				case 'disable':
					$wpdb->update( $wpdb->prefix . Database::ADS_TABLE,
						[
							'enabled' => 0,
						],
						[
							'id' => $id,
						]
					);

					set_transient( 'rpam_quick_action', __('Object successfully disabled', 'rp-ads-manager'), 2 );
					break;
				default:

			}

			header( 'Location: ' . admin_url( 'tools.php?page=rp-ads-manager-plugin' ) );
			exit;
		}

		if ( empty( $_POST ) ) {
			return false;
		}

		$wpView = wpView::getInstance();

		if ( null !== $id && array_key_exists( 'rpam', $_POST ) ) {
			$code  = serialize( $_POST['rpam']['code'] );
			$title = $_POST['rpam']['title'];
			$init  = null;

			if ( array_key_exists( 'use_init', $_POST['rpam'] ) ) {
				$init  = $_POST['rpam']['init'];
			} else {
				$init = NULL;
			}

			$ad = $wpdb->get_row( 'SELECT * FROM ' . $wpdb->prefix . Database::ADS_TABLE . ' WHERE `id` = ' . $id, ARRAY_A );

			$new_add = [];

			if ( $code != $ad['code'] ) {
				$new_add['code'] = $code;
			}
			if ( $title != $ad['title'] ) {
				$new_add['title'] = $title;
			}
			if ( $init != $ad['init'] ) {
				$new_add['init'] = is_null($init) ? NULL : serialize( $init );
			}

			if ( ! empty( $new_add ) ) {
				$wpdb->update( $wpdb->prefix . Database::ADS_TABLE, $new_add, [ 'id' => $id ] );
			}

			if ( $ad['type'] == 'ads' ) {

				$old_options = $wpdb->get_results( 'SELECT `ple_id` AS `id`, `options` FROM ' . $wpdb->prefix . Database::REL_TABLE . ' WHERE `ads_id` = ' . $id, ARRAY_A );

				$places = [];

				foreach ( $_POST['rpam']['display'] as $place => $options ) {
					$places[] = [
						'id'      => $wpView->idFromSlug( $place ),
						'options' => serialize( $options ),
					];
				}

				foreach ( $old_options as $key => $option ) {
					$remove = true;
					foreach ( $places as $one ) {
						if ( $one['id'] == $option['id'] ) {
							$remove = false;
						}
					}
					if ( $remove ) {
						$wpdb->delete( $wpdb->prefix . Database::REL_TABLE, [
							'ads_id' => $id,
							'ple_id' => $option['id']
						] );
					}
				}

				$update_options = array_udiff( $places,
					$old_options,
					function ( $a, $b ) {
						return strcasecmp( serialize( $a ), serialize( $b ) );
					} );

				foreach ( $update_options as $one ) {
					$wpdb->replace( $wpdb->prefix . Database::REL_TABLE,
						[
							'ads_id'  => $id,
							'ple_id'  => $one['id'],
							'options' => $one['options']
						] );
				}
			}

			set_transient( 'rpam_saved', true, 2 );

			header( 'Location: ' . admin_url( 'tools.php?page=rp-ads-manager-plugin&action=editor&type=ads&id=' . $id ) );
			die;

		} elseif ( array_key_exists( 'rpam', $_POST ) ) {
			$type  = $_POST['rpam']['type'];
			$code  = $_POST['rpam']['code'];
			$title = $_POST['rpam']['title'];
			$init  = null;

			if ( array_key_exists( 'use_init', $_POST['rpam'] ) ) {
				$init['code']  = $_POST['rpam']['init']['code'];
				$init['place'] = $_POST['rpam']['init']['place'];
			}

			if ( $type === 'wgt' ) {
				$wpdb->insert(
					$wpdb->prefix . Database::ADS_TABLE,
					[
						'title' => $title,
						'code'  => serialize( $code ),
						'init'  => is_null( $init ) ? $init : serialize( $init ),
						'type'  => 'wgt',
					]
				);

				$inserted_id = $wpdb->insert_id;
			} else {

				$places = [];

				if ( array_key_exists( 'display', $_POST['rpam'] ) ) {
					foreach ( $_POST['rpam']['display'] as $place => $options ) {
						$places[] = [
							'id'      => $wpView->idFromSlug( $place ),
							'options' => $options,
						];
					}
				}

				$wpdb->insert(
					$wpdb->prefix . Database::ADS_TABLE,
					[
						'title' => $title,
						'code'  => serialize( $code ),
						'init'  => is_null( $init ) ? $init : serialize( $init ),
						'type'  => 'ads',
					]
				);

				$inserted_id = $wpdb->insert_id;

				foreach ( $places as $place ) {
					$wpdb->insert(
						$wpdb->prefix . Database::REL_TABLE,
						[
							'ads_id'  => $inserted_id,
							'ple_id'  => $place['id'],
							'options' => serialize( $place['options'] ),
						]
					);
				}
			}

			set_transient( 'rpam_saved', true, 2 );

			header( 'Location: ' . admin_url( 'tools.php?page=rp-ads-manager-plugin&action=editor&type=' . $type . '&id=' . $inserted_id ) );
			exit;
		}

		return false;
	}

	public function editorAction() {
		global $wpdb;

		$record = [
			'id'     => null,
			'title'  => '',
			'code'   => '',
			'init'   => null,
			'type'   => filter_input( INPUT_GET, 'type', FILTER_SANITIZE_STRING ),
			'places' => [],
		];

		/** @var wpView $wpView */
		$wpView = wpView::getInstance();

		if ( array_key_exists( 'id', $_GET ) ) {

			$id = filter_input( INPUT_GET, 'id', FILTER_VALIDATE_INT );

			$ads = $wpdb->prefix . Database::ADS_TABLE;
			$rel = $wpdb->prefix . Database::REL_TABLE;

			$all_records = $wpdb->get_results(
				"SELECT `a`.*, `r`.`options`, `r`.`ple_id`
              FROM $ads AS `a` 
              LEFT JOIN $rel AS `r` ON `r`.`ads_id` = `a`.`id` 
              WHERE `a`.`id` = '$id'
              ORDER BY `a`.`id`",
				ARRAY_A
			);

			$record = [];

			foreach ( $all_records as $one_record ) {
				if ( empty( $record ) ) {
					$record['id']     = $one_record['id'];
					$record['title']  = $one_record['title'];
					$record['code']   = stripslashes( unserialize( $one_record['code'] ) );
					$record['init']   = is_null( $one_record['init'] ) ? null : stripslashes( unserialize( $one_record['init'] ) );
					$record['type']   = $one_record['type'];
					$record['places'] = [];
				}

				$record['places'][ $wpView->getSlug( $one_record['ple_id'] ) ] = unserialize( $one_record['options'] );
			}
		}

		wp_enqueue_code_editor( [ 'type' => 'text/html' ] );
		wp_enqueue_script( 'rp-ads-manager-editor', plugins_url( 'rp-ads-manager/assets/editor.js' ) );

		return [
			'saved'          => get_transient( 'rpam_saved' ),
			'record'         => $record,
			'places'         => $wpView->getOptions(),
			'hasViewOptions' => [
				'after-n-p',
				'after-n-img',
				'after-n-post',
				'after-each',
			],
		];
	}
}