<?php

namespace RP\AdsManager\Common;

use RP\AdsManager\Singleton;

class wpView extends Singleton {
	const SINGLE = 1;
	const PAGE = 2;
	const HOMEPAGE = 3;
	const CATEGORY = 4;
	const ARCHIVE = 5;
	const TAG = 6;

	private static $slug = [
		self::SINGLE   => 'single',
		self::PAGE     => 'page',
		self::HOMEPAGE => 'homepage',
		self::CATEGORY => 'category',
		self::ARCHIVE  => 'archive',
		self::TAG      => 'tag'
	];

	private $options;

	protected function __construct() {

		$positions = [
			'top'    => __( 'at the beginning', 'rp-ads-manager' ),
			'middle' => __( 'in the middle', 'rp-ads-manager' ),
			'bottom' => __( 'in the end', 'rp-ads-manager' ),
		];

		$pagePositions = array_merge(
			$positions,
			[
				'after-n-p'     => __( 'after Paragraph', 'rp-ads-manager' ),
				'before-last-p' => __( 'before last Paragraph', 'rp-ads-manager' ),
				'after-n-img'   => __( 'after Image', 'rp-ads-manager' ),
			]
		);

		$singlePositions = array_merge(
			$pagePositions,
			[
				'after-more' => __( 'after &lt;--more--&gt; tag', 'rp-ads-manager' ),
			]
		);

		$listPositions = array_merge(
			$positions,
			[
				'after-each'   => __( 'after each', 'rp-ads-manager' ),
				'after-n-post' => __( 'after Post', 'rp-ads-manager' ),
			]
		);

		$this->options = [
			self::$slug[ self::SINGLE ]   => [
				'title'     => __( 'Single', 'rp-ads-manager' ),
				'positions' => $singlePositions,
			],
			self::$slug[ self::PAGE ]     => [
				'title'     => __( 'Page', 'rp-ads-manager' ),
				'positions' => $pagePositions,
			],
			self::$slug[ self::HOMEPAGE ] => [
				'title'     => __( 'Homepage', 'rp-ads-manager' ),
				'positions' => $listPositions,
			],
			self::$slug[ self::CATEGORY ] => [
				'title'     => __( 'Category', 'rp-ads-manager' ),
				'positions' => $listPositions,
			],
			self::$slug[ self::ARCHIVE ]  => [
				'title'     => __( 'Archive', 'rp-ads-manager' ),
				'positions' => $listPositions,
			],
			self::$slug[ self::TAG ]      => [
				'title'     => __( 'Tag', 'rp-ads-manager' ),
				'positions' => $listPositions,
			],
		];
	}

	public function idFromSlug( $slug ) {
		$const = wpView::class . '::' . strtoupper( $slug );

		return defined( $const ) ? constant( $const ) : null;
	}

	public function getSlug( $id, $title = false ) {
		return array_key_exists( $id, self::$slug ) ? ( $title ? $this->getTitle( $id ) : self::$slug[ $id ] ) : false;
	}

	private function getTitle( $id ) {
		return __( $this->mb_ucfirst( self::$slug[ $id ] ), 'rp-ads-manager' );
	}

	private function mb_ucfirst( $string ) {
		$string = mb_strtoupper( mb_substr( $string, 0, 1 ) ) . mb_substr( $string, 1 );

		return $string;
	}

	public function getOptions() {
		return $this->options;
	}
}