<?php

namespace RP\AdsManager\Frontend\Hooks;

use RP\AdsManager\Database;
use RP\AdsManager\Frontend\AdInjector;
use RP\AdsManager\Hook;

class Content extends Hook {
	protected $function = 'add_filter';

	protected $args = [
		'the_content',
		[ 'RP\\AdsManager\\Frontend\\Hooks\\Content', 'process' ],
		999,
	];

	public static function process( $content ) {
		if ( ( is_feed() ) || ( false !== strpos( $content, '<!--RPAM-OffAds-->' ) )) {
			return $content;
		}

		$callable = 'process' . ( is_single() ? 'Single' : ( is_page() ? 'Page' : 'Null' ) );

		$content = call_user_func_array( [ 'self', $callable ], [ $content ] );

		return $content;
	}

	private static function processNull( $content ) {
		return $content;
	}

	private static function processPage( $content ) {
		global $wpdb;

		$ads = $wpdb->get_results(
			'SELECT `id`, `code`, `init`, `r`.`options` FROM ' . $wpdb->prefix . Database::ADS_TABLE . ' 
                            LEFT JOIN ' . $wpdb->prefix . Database::REL_TABLE . ' AS `r` ON (`r`.`ads_id` = `id` AND `r`.`ple_id` = 2)  WHERE `type` = \'ads\' AND `enabled` = 1',
			ARRAY_A
		);

		if ( count( $ads ) > 0 ) {
			foreach ( $ads as $ad ) {
				$ad['code']    = stripslashes( unserialize( $ad['code'] ) );
				$ad['options'] = unserialize( $ad['options'] );

				if ( false !== strpos( $content, '<!--rp-ads-' . $ad['id'] . '-->' ) ) {
					$content = str_replace( '<!--rp-ads-' . $ad['id'] . '-->', $ad['code'], $content );
				} else {
					switch ( $ad['options']['position'] ) {
						case 'top':
							$content = AdInjector::before( $content, $ad );
							break;
						case 'middle':
							$content = AdInjector::middle( $content, $ad );
							break;
						case 'bottom':
							$content = AdInjector::after( $content, $ad );
							break;
						case 'after-n-p':
							$content = AdInjector::after( $content, $ad, 'n-p' );
							break;
						case 'before-last-p':
							$content = AdInjector::before( $content, $ad, 'last-p' );
							break;
						case 'after-n-img':
							$content = AdInjector::after( $content, $ad, 'n-img' );
							break;
						default:
					}
				}
			}
		}

		return $content;
	}

	private static function processSingle( $content ) {
		global $wpdb;

		$ads = $wpdb->get_results(
			'SELECT `id`, `code`, `init`, `r`.`options` FROM ' . $wpdb->prefix . Database::ADS_TABLE . ' 
                            LEFT JOIN ' . $wpdb->prefix . Database::REL_TABLE . ' AS `r` ON (`r`.`ads_id` = `id` AND `r`.`ple_id` = 1)  WHERE `type` = \'ads\' AND `enabled` = 1',
			ARRAY_A
		);

		if ( count( $ads ) > 0 ) {
			foreach ( $ads as $ad ) {
				$ad['code']    = stripslashes( unserialize( $ad['code'] ) );
				$ad['options'] = unserialize( $ad['options'] );

				if ( false !== strpos( $content, '<!--rp-ads-' . $ad['id'] . '-->' ) ) {
					$content = str_replace( '<!--rp-ads-' . $ad['id'] . '-->', $ad['code'], $content );
				} else {
					switch ( $ad['options']['position'] ) {
						case 'top':
							$content = AdInjector::before( $content, $ad );
							break;
						case 'middle':
							$content = AdInjector::middle( $content, $ad );
							break;
						case 'bottom':
							$content = AdInjector::after( $content, $ad );
							break;
						case 'after-n-p':
							$content = AdInjector::after( $content, $ad, 'n-p' );
							break;
						case 'before-last-p':
							$content = AdInjector::before( $content, $ad, 'last-p' );
							break;
						case 'after-more':
							$content = AdInjector::after( $content, $ad, 'more' );
							break;
						case 'after-n-img':
							$content = AdInjector::after( $content, $ad, 'n-img' );
							break;
						default:
					}
				}
			}
		}

		return $content;
	}
}